//Microchip Solutions v2011-12-05
/******************************************************************************
            USB Custom Demo, Host

This file provides the main entry point to the Microchip USB Custom
Host demo.  This demo shows how a PIC24F system could be used to
act as the host, controlling a USB device running the Microchip Custom
Device demo.

 ******************************************************************************/

/******************************************************************************
 * Filename:        main.c
 * Dependancies:    USB Host Driver with Generic Client Driver
 * Processor:       PIC24F256GB1xx
 * Hardware:        Explorer 16 with USB PICtail Plus
 * Compiler:        C30 v2.01/C32 v0.00.18
 * Company:         Microchip Technology, Inc.

Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the ?ompany? for its PICmicro? Microcontroller is intended and
supplied to you, the Company? customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN ?S IS?CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


 *******************************************************************************/
//0x0004 = Attribute Protocol
//0x0005 = LE L2CAP Signaling channel
//0x0006 = Security Manager Protocol

#include "mcc_generated_files/system.h"
#include "mcc_generated_files/usb/usb.h"
#include "mcc_generated_files/uart2.h"
#include "mcc_generated_files/memory/flash.h"
#include "mcc_generated_files/led.h"
#include "mcc_generated_files/connectswitch.h"

// *****************************************************************************
// *****************************************************************************
// Internal Function Prototypes
// *****************************************************************************
// *****************************************************************************

void DelayMs(uint16_t);

// *****************************************************************************
// *****************************************************************************
// Macros
// *****************************************************************************
// *****************************************************************************

//#define DEBUG_MODE

#define MAX_NO_OF_BYTES  64

#define DISCONNECT 0x05
#define ENCRYPT_EVENT 0x08
//#define NUM_OF_COMP_PACKES_EVENT 0x13
#define BL_EVENT 0x0e
#define BLE_EVENT 0x3e

// *****************************************************************************
// *****************************************************************************
// Type definitions
// *****************************************************************************
// *****************************************************************************


// Application States
typedef enum {
    BT_INITIALIZE = 0, // Initialize the app when a device is attached
    BT_STATE_IDLE, // Inactive State
    BT_STATE_PROCESS,
    BT_STATE_WRITE_CLASS,
    BT_STATE_READ_EP1,
    BT_STATE_READ_CLASS_WAITING,
    BT_STATE_WRITE_ACL,
    BT_STATE_READ_ACL_HCI,
    BT_STATE_READ_ACL_WAITING,
    BT_STATE_READ_HCI_WAITING,
    BT_STATE_READ_HCI,

    BT_STATE_ERROR // An error has occured
} BT_STATE;


// Hci States
typedef enum {
    HCI_CMD_RESET = 0, // Initialize the hci when a device is attached
    HCI_CMD_RESET_END,
    HCI_CMD_READ_BD_ADDR,
    HCI_CMD_READ_BD_ADDR_END,
    HCI_CMD_MASK,
    HCI_CMD_MASK_END,

    HCI_CMD_ADVERT_DATA,
    HCI_CMD_ADVERT_DATA_END,
    HCI_CMD_ADVERT_PARAM,
    HCI_CMD_ADVERT_PARAM_END,
    HCI_CMD_ADVERT_ENABLE,
    HCI_CMD_ADVERT_ENABLE_END,

    L2CAP_DISCONNECT_REQ,
    L2CAP_DISCONNECT_REQ_END,

    GATT_SARVICE_CHANGE,
    GATT_MTU_RES,
    GATT_GEN_ATT_SERVICE,
    GATT_GEN_ATT_SERVICE_RES,
    GATT_PRIMARY_SERVICE1,
    GATT_PRIMARY_SERVICE1_RES,
    GATT_PRIMARY_SERVICE2,
    GATT_PRIMARY_SERVICE2_RES,

    GATT_DEVICE_INDUCED_SERVICE,
    GATT_DEVICE_INDUCED_SERVICE_RES,
    GATT_DEVICE_CHAR1_REQ,
    GATT_DEVICE_CHAR1_RES,
    GATT_DEVICE_CHAR2_REQ,
    GATT_DEVICE_CHAR2_RES,
    GATT_PNP_REQ,
    GATT_PNP_RES,
    GATT_DVICE_INFO_REQ,
    GATT_DEVICE_INFO_RES,

    GATT_BATTERY_INDUCED_SERVICE,
    GATT_BATTERY_INDUCED_SERVICE_RES,
    GATT_BATTERY_CHAR1_REQ,
    GATT_BATTERY_CHAR1_RES,
    GATT_BATTERY_CHAR2_REQ,
    GATT_BATTERY_CHAR2_RES,
    GATT_BATTERY_NOW_REQ,
    GATT_BATTERY_NOW_RES,
    GATT_BATTERY_DESC_REQ,
    GATT_BATTERY_DESC_RES,
    GATT_BATTERY_CCC_REQ,
    GATT_BATTERY_CCC_RES,

    GATT_HID_INDUCED_SERVICE,
    GATT_HID_INDUCED_SERVICE_RES,
    GATT_HID_CHAR1_REQ,
    GATT_HID_CHAR1_RES,
    GATT_HID_CHAR2_REQ,
    GATT_HID_CHAR2_RES,
    GATT_HID_CHAR3_REQ,
    //		GATT_HID_CHAR3_RES,
    //	GATT_HID_CHAR4_REQ,
    GATT_HID_CHAR4_RES,
    GATT_HID_INFO_REQ,
    GATT_HID_INFO_RES,
    GATT_HID_BOOT_IN_REQ,
    GATT_HID_BOOT_IN_RES,
    //	GATT_HID_BOOT_OUT_REQ,
    //		GATT_HID_BOOT_OUT_RES,
    GATT_HID_DESC1_REQ,
    GATT_HID_DESC1_RES,
    GATT_HID_CCC1_REQ,
    GATT_HID_CCC1_RES,
    GATT_HID_REPORT1_REQ,
    GATT_HID_REPORT1_RES,
    GATT_HID_REPORT2_REQ,
    GATT_HID_REPORT2_RES,
    GATT_HID_REPORT_REQ3,
    GATT_HID_REPORT_RES3,
    GATT_HID_DATA_REQ,
    GATT_HID_DATA_RES,
    GATT_HID_DESC2_REQ,
    GATT_HID_DESC2_RES,
    GATT_HID_CCC2_REQ,
    GATT_HID_CCC2_RES,
    GATT_REPORT_ID_IN_REQ,
    GATT_REPORT_ID_IN_RES,
    GATT_PROT_MODE_REQ,
    GATT_PROT_MODE_RES,

    GATT_ACCESS_INDUCED_SERVICE,
    GATT_ACCESS_INDUCED_SERVICE_RES,
    GATT_ACCESS_CHAR1_REQ,
    GATT_ACCESS_CHAR1_RES,
    GATT_ACCESS_CHAR2_REQ,
    GATT_ACCESS_CHAR2_RES,
    GATT_ACCESS_ICON_REQ,
    GATT_ACCESS_ICON_RES,
    GATT_ACCESS_NAME_REQ,
    GATT_ACCESS_NAME_RES,

    GATT_BATTERY_NOW2_REQ,
    GATT_BATTERY_NOW2_RES,

    //for re-connection
    SECURITY_REQ,
    WAIT_LTK_REQ_EVENT,
    LTK_REQ_REPLY,
    GATT_MTU_REQ_RE,
    GATT_MTU_RES_RE,
    GATT_MTU_RES_RE_HCI,
    //        GATT_BATTERY_NOW_INFO,

    CONNECT_PARAM_UPDATE_REQ,
    CONNECT_PARAM_UPDATE1_RES,
    GATT_BATTERY_NOW_RE_RES,
    CONNECT_PARAM_UPDATE2_RES,
    //

    SMP_PAIR_REQ,
    SMP_PAIR_RES,
    SMP_FUNC_E,
    SMP_FUNC_E_END,
    SMP_FUNC_C1,
    SMP_FUNC_C1_END,
    SMP_FUNC_S1,
    SMP_FUNC_S1_END,
    SMP_PAIR_CONF_IN,
    SMP_PAIR_CONF_OUT,
    SMP_RANDOM_IN,
    SMP_RANDOM_OUT,
    //	SMP_RANDOM1_IN,
    SMP_RANDOM1_OUT,
    SMP_LTK_REQ,
    SMP_LTK_RES,
    SMP_LTK_RES_END,
    SMP_LTK_OUT,
    //		SMP_LTK_IN,

    //	HID_READ_DATA,
    //	HID_WRITE_RES,
    HID_WRITE_DATA,
    //	HID_WRITE_DATA1,
    //	HID_WRITE_DATA2,
    //	HID_WRITE_DATA3,
    HID_READ_HCI,
    PROG_END
} HCI_STATE;

// *****************************************************************************
// *****************************************************************************
// Local Variables
// *****************************************************************************
// *****************************************************************************

// Allocate and reserve a page of flash for this test to use.  The compiler/linker will reserve this for data and not place any code here.
static __prog__ uint8_t flashTestPage[FLASH_ERASE_PAGE_SIZE_IN_PC_UNITS] __attribute__((space(prog), aligned(FLASH_ERASE_PAGE_SIZE_IN_PC_UNITS)));
static uint32_t flash_storage_address;
static uint32_t write_data[2] = {0};
static uint32_t read_data[2] = {0};

uint8_t deviceAddress; // Address of the device on the USB
BT_STATE DemoState; // Current state of the demo application
HCI_STATE HciState; // Current state of the demo application
uint16_t data_num;

uint8_t buf[MAX_NO_OF_BYTES];
uint8_t buf1[MAX_NO_OF_BYTES];
uint8_t handle[2]; //a handle for ACL
uint8_t EP2busy_flag = 0;

uint16_t step;
char message[30];
uint16_t end_num;

uint8_t PAIR_CONF[16];
uint8_t PAIR_RANDOM[16] = {0x94, 0x58, 0x7f, 0x05, 0xdc, 0x5d, 0x8f, 0xe6, 0xdd, 0x26, 0x3e, 0x6d, 0x2b, 0x16, 0xb4, 0x7a};
uint8_t INIT_RANDOM[16];
uint8_t STK[16];

uint8_t INI_ADDRESS_TYPE = 0, RES_ADDRESS_TYPE = 0;
uint8_t INI_BT_ADDRESS[6], RES_BT_ADDRESS[6];
uint8_t PAIR_RES[7] = {0x02, 0x03, 0x00, 0x01, 0x10, 0x01, 0x01}, PAIR_REQ[7];
uint8_t P1[16], P2[16];

uint8_t EDIV_RANDOM[10] = {0x3C, 0x03, 0x5D, 0x83, 0xA8, 0xDF, 0x97, 0xE4, 0xA1, 0xC2};
uint8_t LTK[16] = {0xA4, 0x21, 0x5D, 0xB1, 0xEE, 0x4D, 0x1A, 0x74, 0xD2, 0xB8, 0x47, 0xCA, 0xDE, 0xBB, 0xEE, 0xE9};
uint8_t M_EDIV_RANDOM[10];
uint8_t M_LTK[16];

uint8_t HID_CHAR[7][7] = {
    {0x07, 0x00, 0x06, 0x08, 0x00, 0x4e, 0x2a},
    {0x09, 0x00, 0x12, 0x0a, 0x00, 0x4d, 0x2a},
    {0x0d, 0x00, 0x02, 0x0e, 0x00, 0x4b, 0x2a},
    {0x0f, 0x00, 0x12, 0x10, 0x00, 0x33, 0x2a},
    {0x12, 0x00, 0x02, 0x13, 0x00, 0x4a, 0x2a},
    {0x14, 0x00, 0x04, 0x15, 0x00, 0x4c, 0x2a},
    {0x08, 0x15, 0x00, 0x0a, 0xFF, 0xFF, 0xFF}
};

uint8_t HID_REPORT[3][22] = {
    {0x05, 0x01, 0x09, 0x02, 0xa1, 0x01, 0x09, 0x01, 0xa1, 0x00, 0x05, 0x09, 0x19, 0x01, 0x29,
        0x03, 0x15, 0x00, 0x25, 0x01, 0x95, 0x03},
    {0x75, 0x01, 0x81, 0x02, 0x95, 0x01, 0x75, 0x05, 0x81, 0x01, 0x05, 0x01, 0x09, 0x30, 0x09,
        0x31, 0x15, 0x81, 0x25, 0x7f, 0x75, 0x08},
    {0x95, 0x02, 0x81, 0x06, 0xc0, 0xc0, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}
};


uint8_t ACCESS_SERVICE[3][7] = {
    {0x02, 0x00, 0x0a, 0x03, 0x00, 0x00, 0x2a},
    {0x04, 0x00, 0x02, 0x05, 0x00, 0x01, 0x2a},
    {0x08, 0x05, 0x00, 0x0a, 0xFF, 0xFF, 0xFF}
};

uint8_t PNP_SERVICE[2][7] = {
    {0x1b, 0x00, 0x02, 0x1c, 0x00, 0x50, 0x2a},
    {0x08, 0x1c, 0x00, 0x0a, 0xFF, 0xFF, 0xFF}
};

bool ConnectionSW;

// data for Mouse
uint8_t movement_length;
uint8_t vector = 0;
//The direction that the mouse will move in
int8_t dir_table[] = {-4, -4, -4, 0, 4, 4, 4, 0};
uint8_t mouse[3];

//******************************************************************************
//******************************************************************************
// Local Routines
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        CheckForNewAttach
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          deviceAddress (global)
 *                  Updates the device address when an attach is found.
 *
 * Returns:         true if a new device has been attached.  false,
 *                  otherwise.
 *
 * Side Effects:    Prints attach message
 *
 * Overview:        This routine checks to see if a new device has been
 *                  attached.  If it has, it records the address.
 *************************************************************************/

bool CheckForNewAttach(void) {
    // Try to get the device address, if we don't have one.
    if (deviceAddress == 0) {
        GENERIC_DEVICE_ID DevID;

        DevID.vid = 0x04D8; //NO meaning: Any number is OK.
        DevID.pid = 0x000C; //NO meaning: Any number is OK.
#ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
        DevID.serialNumberLength = 0;
        DevID.serialNumber = NULL;
#endif

        if (USBHostGenericGetDeviceAddress(&DevID)) {
            deviceAddress = DevID.deviceAddress;
            UART2PrintString("Generic demo device attached - polled, deviceAddress=");
            UART2PutDec(deviceAddress);
            UART2PrintString("\r\n");
            return true;
        }
    }

    return false;

} // CheckForNewAttach

/*************************************************************************
 * Function:        ManageDemoState
 *
 * Preconditions:   The DemoState global variable must be initialized to
 *                  DEMO_STATE_IDLE (0).  (This occurs on reset.)
 *
 * Input:           DemoState (global)
 *                  Actions selected based value of DemoState on function
 *                  entry.
 *
 *                  deviceAddress (global)
 *                  May use device address to access device, depending on
 *                  state.
 *
 *                  DataPacket (global)
 *                  May read data from packet buffer, depending on state.
 *
 * Output:          DemoState (global)
 *                  Updates demo state as appropriate.
 *
 *                  DataPacket (global)
 *                  May cause data in the packet buffer to be updated,
 *                  depending on state.
 *
 * Returns:         None
 *
 * Side Effects:    Depend on state transition
 *
 * Overview:        This routine maintains the state of the application,
 *                  updateing global data and taking actions as necessary
 *                  to maintain the custom demo operations.
 *************************************************************************/
void ManageDemoState(void) {
    uint8_t RetVal;

    if (USBHostGenericDeviceDetached(deviceAddress) && deviceAddress != 0) {
        UART2PrintString("Generic demo device detached - polled\r\n");
        DemoState = BT_INITIALIZE;
        HciState = HCI_CMD_RESET;
        deviceAddress = 0;
    }

    switch (DemoState) {
        case BT_INITIALIZE:
            ConnectionSW = CONNECTSWITCH_IsPressed();
            DemoState = BT_STATE_IDLE;
            break;

            /** Idle State:  Loops here until attach **/
        case BT_STATE_IDLE:
            if (CheckForNewAttach()) {
                DemoState = BT_STATE_PROCESS;
                HciState = HCI_CMD_RESET;
            }
            break;

        case BT_STATE_PROCESS:
            switch (HciState) {
                    //HCI layer***********************************************************************
                case HCI_CMD_RESET:
                    buf1[0] = 0x03;
                    buf1[1] = 0x0c;
                    buf1[2] = 0;

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = HCI_CMD_RESET_END;
                    break;

                case HCI_CMD_RESET_END:
                    end_num = BL_EVENT; // When buf1[0]==end_num, reading process ends. See BT_STATE_READ_EP1
                    strcpy(message, "HCI_CMD_RESET: "); //message for BT_STATE_READ_EP1
                    DemoState = BT_STATE_READ_EP1;
                    HciState = HCI_CMD_READ_BD_ADDR;
                    break;

                    //********************************************************************************
                case HCI_CMD_READ_BD_ADDR:
                    buf1[0] = 0x09;
                    buf1[1] = 0x10;
                    buf1[2] = 0;

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = HCI_CMD_READ_BD_ADDR_END;
                    break;

                case HCI_CMD_READ_BD_ADDR_END:
                    end_num = BL_EVENT;
                    strcpy(message, "HCI_CMD_READ_BD_ADDR: ");
                    DemoState = BT_STATE_READ_EP1;
                    HciState = HCI_CMD_MASK;
                    break;

                    //********************************************************************************
                case HCI_CMD_MASK:

                    for (step = 0; step < 6; step++) RES_BT_ADDRESS[step] = buf1[6 + step];

#ifdef DEBUG_MODE
                    for (data_num = 0; data_num < 6; data_num++) {
                        UART2PutHex(RES_BT_ADDRESS[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");
#endif

                    buf1[0] = 0x01;
                    buf1[1] = 0x0c;
                    buf1[2] = 0x08;
                    buf1[3] = 0xbf; //af;//2f; + Encryption Change Event + Disconnection
                    buf1[4] = 0xff;
                    buf1[5] = 0xff;
                    buf1[6] = 0xff;
                    buf1[7] = 0xff;
                    buf1[8] = 0xff;
                    buf1[9] = 0xbf;
                    buf1[10] = 0x3d;

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = HCI_CMD_MASK_END;
                    break;

                case HCI_CMD_MASK_END:
                    end_num = BL_EVENT;
                    strcpy(message, "HCI_CMD_MASK: ");
                    DemoState = BT_STATE_READ_EP1;
                    HciState = HCI_CMD_ADVERT_DATA;
                    break;

                    //********************************************************************************
                case HCI_CMD_ADVERT_DATA:
                    //LE Set Advertising Data Command
                    buf1[0] = 0x08;
                    buf1[1] = 0x20;
                    buf1[2] = 0x20; //PDU length
                    buf1[3] = 0x13; // data length 3+4+4+8=19

                    buf1[4] = 0x02; //length
                    buf1[5] = 0x01; //flags
                    buf1[6] = 0x05;
                    buf1[7] = 0x03; //length
                    buf1[8] = 0x03; //UUID
                    buf1[9] = 0x12; //0x1812 HID service
                    buf1[10] = 0x18;
                    buf1[11] = 0x03; //length
                    buf1[12] = 0x19;
                    buf1[13] = 0xc2; //appearance(Mouse)
                    buf1[14] = 0x03;
                    buf1[15] = 0x07; //length
                    buf1[16] = 0x09; //local name
                    buf1[17] = 'Y';
                    buf1[18] = 'T';
                    buf1[19] = 'M';
                    buf1[20] = '-';
                    buf1[21] = 'L';
                    buf1[22] = 'E';
                    for (step = 23; step < 35; step++)
                        buf1[step] = 0x00;

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = HCI_CMD_ADVERT_DATA_END;
                    break;

                case HCI_CMD_ADVERT_DATA_END:
                    end_num = BL_EVENT;
                    strcpy(message, "HCI_CMD_SCAN_ENABLE: ");
                    DemoState = BT_STATE_READ_EP1;
                    HciState = HCI_CMD_ADVERT_PARAM;
                    break;

                    //********************************************************************************
                case HCI_CMD_ADVERT_PARAM:
                    //LE Set Advertising Parameters Command
                    buf1[0] = 0x06;
                    buf1[1] = 0x20;
                    buf1[2] = 0x0f; //length
                    buf1[3] = 0x00; //0xa0;// Advertising_Interval_Min
                    buf1[4] = 0x01; //0x08;//0x00;
                    buf1[5] = 0x00; //0x00;//Advertising_Interval_Max
                    buf1[6] = 0x01; //0x08;//0x40;

                    //Advertising_Type
                    if (ConnectionSW) buf1[7] = 0x00; //Connectable undirected advertising
                    else buf1[7] = 0x01; //Connectable directed advertising
                    // buf1[7] = 0x00; //Connectable undirected advertising

                    //Own_Address_Type
                    buf1[8] = 0x00; // Public Device Address(default)


                    //Direct_Address_Type
                    buf1[9] = 0x00; //Direct_Address_Type

                    if (!ConnectionSW) {
                        ////////////////// NVM
                        // Get flash page aligned address of flash reserved above for this test.
                        flash_storage_address = FLASH_GetErasePageAddress((uint32_t) & flashTestPage[0]);

                        read_data[0] = FLASH_ReadWord24(flash_storage_address);
                        read_data[1] = FLASH_ReadWord24(flash_storage_address + 2U);

                        buf1[12] = (uint8_t) (read_data[0] & 0x000000ff);
                        buf1[11] = (uint8_t) ((read_data[0] >> 8) & 0x000000ff);
                        buf1[10] = (uint8_t) ((read_data[0] >> 16) & 0x000000ff);
                        buf1[15] = (uint8_t) (read_data[1] & 0x000000ff);
                        buf1[14] = (uint8_t) ((read_data[1] >> 8) & 0x000000ff);
                        buf1[13] = (uint8_t) ((read_data[1] >> 16) & 0x000000ff);
                        //////////////////
                    } else {
                        buf1[10] = 0x00; //Direct_Address
                        buf1[11] = 0x00;
                        buf1[12] = 0x00;
                        buf1[13] = 0x00;
                        buf1[14] = 0x00;
                        buf1[15] = 0x00;
                    }
                    buf1[16] = 0x07; //Advertising_Channel_Map
                    buf1[17] = 0x00; //Advertising_Filter_Policy

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = HCI_CMD_ADVERT_PARAM_END;
                    break;

                case HCI_CMD_ADVERT_PARAM_END:
                    end_num = BL_EVENT;
                    strcpy(message, "HCI_CMD_ADVERT_PARAM: ");
                    DemoState = BT_STATE_READ_EP1;
                    HciState = HCI_CMD_ADVERT_ENABLE;
                    break;

                    //********************************************************************************
                case HCI_CMD_ADVERT_ENABLE:
                    //Set Advertise Enable Command
                    buf1[0] = 0x0a;
                    buf1[1] = 0x20;
                    buf1[2] = 0x01; //length
                    buf1[3] = 0x01; //Advertising is enabled.

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = HCI_CMD_ADVERT_ENABLE_END;
                    break;

                case HCI_CMD_ADVERT_ENABLE_END:
                    end_num = BLE_EVENT;
                    strcpy(message, "HCI_CMD_ADVERT_ENABLE: ");
                    DemoState = BT_STATE_READ_EP1;

                    if (ConnectionSW)
                        HciState = GATT_SARVICE_CHANGE; // GO TO FIRST CONNECTION STAGE
                    else
                        HciState = SECURITY_REQ; // GO TO RE-CONNECTION STAGE
                    break;

                    //********************************************************************************
                    //********************************************************************************
                    // RE-CONNECTION STAGE
                    //********************************************************************************
                    //********************************************************************************
                case SECURITY_REQ:
                    if (buf1[3] == 0x3c) {
                        //RE-CONNECTION FAILED
                        HciState = HCI_CMD_ADVERT_ENABLE;
                        UART2PrintString("re-");
                        break;
                    }

                    handle[0] = buf1[4];
                    handle[1] = buf1[5];

                    buf[0] = handle[0];
                    buf[1] = handle[1] + 0x20;
                    buf[2] = 0x06;
                    buf[3] = 0x00;
                    buf[4] = 0x02;
                    buf[5] = 0x00;
                    buf[6] = 0x06;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x01;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = WAIT_LTK_REQ_EVENT;

                    break;

                case WAIT_LTK_REQ_EVENT:
                    end_num = BLE_EVENT;
                    strcpy(message, "WAIT_LTK_REQ_EVENT:\r\n");
                    DemoState = BT_STATE_READ_EP1;
                    HciState = LTK_REQ_REPLY;
                    break;

                    //********************************************************************************
                case LTK_REQ_REPLY:

                    if (buf1[2] != 0x05) {//if is not Long Term Key Request Event
                        HciState = WAIT_LTK_REQ_EVENT;
                        break;
                    }

                    buf1[0] = 0x01a;
                    buf1[1] = 0x20;
                    buf1[2] = 0x12;
                    //buf1[3]=0x2f;
                    //buf1[4]=0xff;
                    for (step = 0; step < 16; step++) buf1[5 + step] = LTK[step];

                    UART2PrintString("START LTK ------ !\r\n");

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = GATT_MTU_REQ_RE;
                    break;

                case GATT_MTU_REQ_RE:
                    end_num = BL_EVENT;
                    strcpy(message, "GATT_MTU_REQ_RE:\r\n ");
                    //DemoState = BT_STATE_READ_EP1;
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_MTU_RES_RE;
                    break;

                    //********************************************************************************
                case GATT_MTU_RES_RE:

                    buf[2] = 0x07;
                    buf[3] = 0x00;
                    buf[4] = 0x03;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x03;
                    buf[9] = 0x17;
                    buf[10] = 0x00;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_MTU_RES_RE_HCI;
                    break;

                case GATT_MTU_RES_RE_HCI:
                    DemoState = BT_STATE_READ_HCI;
                    HciState = CONNECT_PARAM_UPDATE_REQ;
                    break;

                    //********************************************************************************
                    /*
                            case    GATT_BATTERY_NOW_INFO:

                                if(buf1[1]==0x0d) {
                                    //RE-CONNECTION FAILED
                                    while(1){
                                        DelayMs(50);
                                        LATBbits.LATB15=1;
                                        DelayMs(50);
                                        LATBbits.LATB15=0;
                                    }
                                }
                                buf[2]=0x08;
                                buf[3]=0x00;
                                buf[4]=0x04;
                                buf[5]=0x00;
                                buf[6]=0x04;
                                buf[7]=0x00;
                                buf[8]=0x1b;
                                buf[9]=0x1a;
                                buf[10]=0x00;
                                buf[11]=0x32; // battery 50 %


                                DemoState = BT_STATE_WRITE_ACL;
                                HciState =  CONNECT_PARAM_UPDATE_REQ;
                                break;

                     */
                    //********************************************************************************
                case CONNECT_PARAM_UPDATE_REQ:

                    buf[2] = 0x10;
                    buf[3] = 0x00;
                    buf[4] = 0x0c;
                    buf[5] = 0x00;
                    buf[6] = 0x05;
                    buf[7] = 0x00;
                    buf[8] = 0x12;
                    buf[9] = 0x01;
                    buf[10] = 0x08;
                    buf[11] = 0x00;
                    buf[12] = 0x06; //0x0a;
                    buf[13] = 0x00;
                    buf[14] = 0x06; //0x0a;
                    buf[15] = 0x00;
                    buf[16] = 0x01; //0x64;
                    buf[17] = 0x00;
                    buf[18] = 0xe2;
                    buf[19] = 0x04;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = CONNECT_PARAM_UPDATE1_RES;
                    break;

                case CONNECT_PARAM_UPDATE1_RES:
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_BATTERY_NOW_RE_RES;
                    break;

                    //********************************************************************************
                case GATT_BATTERY_NOW_RE_RES:
                    if (buf[6] != 0x04) {

                        buf[2] = 0x08; //length
                        buf[3] = 0x00;
                        buf[4] = 0x04;
                        buf[5] = 0x00;
                        buf[6] = 0x04;
                        buf[7] = 0x00;
                        buf[8] = 0x1b; //battery indication
                        buf[9] = 0x18;
                        buf[10] = 0x00;
                        buf[11] = 0x32;

                        DemoState = BT_STATE_WRITE_ACL;
                        HciState = HID_WRITE_DATA;
                        break;

                    }
                    buf[2] = 0x06; //length
                    buf[3] = 0x00;
                    buf[4] = 0x02;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x32;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = CONNECT_PARAM_UPDATE2_RES;
                    break;

                case CONNECT_PARAM_UPDATE2_RES:
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = HID_WRITE_DATA;
                    break;

                    //********************************************************************************
                    //********************************************************************************
                    // FIRST CONNECTION STAGE
                    //********************************************************************************
                    //********************************************************************************
                case GATT_SARVICE_CHANGE:

                    handle[0] = buf1[4];
                    handle[1] = buf1[5];

                    INI_ADDRESS_TYPE = buf1[7];
                    for (step = 0; step < 6; step++) INI_BT_ADDRESS[step] = buf1[8 + step];


                    for (step = 0; step < 6; step++) {
                        UART2PutHex(buf[step]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");

                    ////////////////// NVM
                    // Get flash page aligned address of flash reserved above for this test.
                    flash_storage_address = FLASH_GetErasePageAddress((uint32_t) & flashTestPage[0]);

                    // Program Valid Key for NVM Commands
                    FLASH_Unlock(FLASH_UNLOCK_KEY);

                    // Erase the page of flash at this address
                    FLASH_ErasePage(flash_storage_address);

                    write_data[0] = ((uint32_t) buf1[8] << 16) + ((uint32_t) buf1[9] << 8) + (uint32_t) buf1[10];
                    write_data[1] = ((uint32_t) buf1[11] << 16) + ((uint32_t) buf1[12] << 8) + (uint32_t) buf1[13];

                    // Write 24 bit Data to the first location.
                    FLASH_WriteWord24(flash_storage_address, write_data[0]);
                    FLASH_WriteWord24(flash_storage_address + 2U, write_data[1]);

                    // Clear Key for NVM Commands so accidental call to flash routines will not corrupt flash
                    FLASH_Lock();
                    //////////////////

#ifdef DEBUG_MODE
                    for (data_num = 0; data_num < 6; data_num++) {
                        UART2PutHex(INI_BT_ADDRESS[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");
#endif

                    strcpy(message, "GATT_MTU_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = SMP_PAIR_REQ;
                    //GOTO SMP
                    break;

                    //********************************************************************************
                case GATT_MTU_RES:
                    //RETUEN FROM SMP
                    buf[2] = 0x07;
                    buf[3] = 0x00;
                    buf[4] = 0x03;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x03;
                    buf[9] = 0x17;
                    buf[10] = 0x00;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_GEN_ATT_SERVICE;
                    break;

                case GATT_GEN_ATT_SERVICE:
                    strcpy(message, "GATT_GEN_ATT_SERVICE_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_GEN_ATT_SERVICE_RES;
                    break;

                    //********************************************************************************
                case GATT_GEN_ATT_SERVICE_RES:

                    buf[2] = 0x09;
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x01;
                    buf[9] = 0x06;
                    buf[10] = 0x01;
                    buf[11] = 0x00;
                    buf[12] = 0x0a;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_PRIMARY_SERVICE1;
                    break;

                case GATT_PRIMARY_SERVICE1:
                    strcpy(message, "GATT_PRIMARY_SERVICE1:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_PRIMARY_SERVICE1_RES; //GO TO FIRST CONNECTION
                    break;

                    //********************************************************************************
                case GATT_PRIMARY_SERVICE1_RES:

                    buf[2] = 0x18;
                    buf[3] = 0x00;
                    buf[4] = 0x14;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x11; //Read by Group Type Response
                    buf[9] = 0x06; //lendth par type
                    buf[10] = 0x01; //handle
                    buf[11] = 0x00;
                    buf[12] = 0x05; //end group (from 0x0001 to 0x0007
                    buf[13] = 0x00;
                    buf[14] = 0x00; //type
                    buf[15] = 0x18;
                    buf[16] = 0x06; //handle
                    buf[17] = 0x00;
                    buf[18] = 0x15; //end group (from 0x0008 to 0x0008)
                    buf[19] = 0x00;
                    buf[20] = 0x12; //type
                    buf[21] = 0x18;
                    buf[22] = 0x16; //handle
                    buf[23] = 0x00;
                    buf[24] = 0x19; //0x1c;//end group (from 0x0009 to 0x001c)
                    buf[25] = 0x00;
                    buf[26] = 0x0f; //type
                    buf[27] = 0x18;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_PRIMARY_SERVICE2;
                    break;

                case GATT_PRIMARY_SERVICE2://0x2800:GATT Primary Service Declaration
                    strcpy(message, "GATT_PRIMARY_SERVICE2:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_PRIMARY_SERVICE2_RES;
                    break;

                    //********************************************************************************
                case GATT_PRIMARY_SERVICE2_RES:

                    buf[2] = 0x0c;
                    buf[3] = 0x00;
                    buf[4] = 0x08;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x11; //Read by Group Type Response
                    buf[9] = 0x06; //lendth par type
                    buf[10] = 0x1a; //handle
                    buf[11] = 0x00;
                    buf[12] = 0xff; //end group
                    buf[13] = 0xff;
                    buf[14] = 0x0a; //type
                    buf[15] = 0x18;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_DEVICE_INDUCED_SERVICE;
                    break;

                case GATT_DEVICE_INDUCED_SERVICE:
                    strcpy(message, "GATT_DEVICE_INDUCED_SERVICE:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_DEVICE_INDUCED_SERVICE_RES;
                    break;

                    //********************************************************************************
                case GATT_DEVICE_INDUCED_SERVICE_RES:

                    buf[2] = 0x09;
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x01;
                    buf[9] = 0x08;
                    buf[10] = 0x1a;
                    buf[11] = 0x00;
                    buf[12] = 0x0a;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_DEVICE_CHAR1_REQ;
                    break;


                case GATT_DEVICE_CHAR1_REQ:
                    strcpy(message, "GATT_DEVICE_CHAR1_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_DEVICE_CHAR1_RES;
                    break;

                    //********************************************************************************
                case GATT_DEVICE_CHAR1_RES:

                    buf[2] = 0x0d; //length
                    buf[3] = 0x00;
                    buf[4] = 0x09;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x09;
                    buf[9] = 0x07;
                    buf[10] = 0x1b;
                    buf[11] = 0x00;
                    buf[12] = 0x02;
                    buf[13] = 0x1c;
                    buf[14] = 0x00;
                    buf[15] = 0x50;
                    buf[16] = 0x2a;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_DEVICE_CHAR2_REQ;
                    break;

                case GATT_DEVICE_CHAR2_REQ:
                    strcpy(message, " GATT_DEVICE_CHAR2_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_DEVICE_CHAR2_RES;
                    break;

                    //********************************************************************************
                case GATT_DEVICE_CHAR2_RES:

                    buf[2] = 0x09;
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x01;
                    buf[9] = 0x08;
                    buf[10] = 0x1c;
                    buf[11] = 0x00;
                    buf[12] = 0x0a;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_PNP_REQ;
                    break;

                case GATT_PNP_REQ:
                    strcpy(message, "GATT_PNP_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_PNP_RES;
                    break;

                    //********************************************************************************
                case GATT_PNP_RES:

                    buf[2] = 0x0c; //length
                    buf[3] = 0x00;
                    buf[4] = 0x08;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x01;
                    buf[10] = 0xd8;
                    buf[11] = 0x04;
                    buf[12] = 0x00;
                    buf[13] = 0x00;
                    buf[14] = 0x11;
                    buf[15] = 0x01;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_DVICE_INFO_REQ;
                    break;

                case GATT_DVICE_INFO_REQ:
                    strcpy(message, "GATT_DVICE_INFO_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_DEVICE_INFO_RES;
                    break;

                    //********************************************************************************
                case GATT_DEVICE_INFO_RES:

                    buf[2] = 0x09;
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x01;
                    buf[9] = 0x04;
                    buf[10] = 0x1d;
                    buf[11] = 0x00;
                    buf[12] = 0x0a;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_BATTERY_INDUCED_SERVICE;
                    break;

                case GATT_BATTERY_INDUCED_SERVICE:
                    strcpy(message, "GATT_BATTERY_INDUCED_SERVICE:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_BATTERY_INDUCED_SERVICE_RES;
                    break;

                    //********************************************************************************
                case GATT_BATTERY_INDUCED_SERVICE_RES:

                    buf[2] = 0x09;
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x01;
                    buf[9] = 0x08;
                    buf[10] = 0x16;
                    buf[11] = 0x00;
                    buf[12] = 0x0a;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_BATTERY_CHAR1_REQ;
                    break;

                case GATT_BATTERY_CHAR1_REQ:
                    strcpy(message, "GATT_BATTERY_CHAR1_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_BATTERY_CHAR1_RES;
                    break;

                    //********************************************************************************
                case GATT_BATTERY_CHAR1_RES:

                    buf[2] = 0x0d; //length
                    buf[3] = 0x00;
                    buf[4] = 0x09;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x09;
                    buf[9] = 0x07;
                    buf[10] = 0x17;
                    buf[11] = 0x00;
                    buf[12] = 0x12;
                    buf[13] = 0x18;
                    buf[14] = 0x00;
                    buf[15] = 0x19;
                    buf[16] = 0x2a;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_BATTERY_CHAR2_REQ;
                    break;

                case GATT_BATTERY_CHAR2_REQ:
                    strcpy(message, "GATT_BATTERY_CHAR2_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_BATTERY_CHAR2_RES;
                    break;

                    //********************************************************************************
                case GATT_BATTERY_CHAR2_RES:

                    buf[2] = 0x09; //length
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x01;
                    buf[9] = 0x08;
                    buf[10] = 0x18;
                    buf[11] = 0x00;
                    buf[12] = 0x0a;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_BATTERY_NOW_REQ;
                    break;

                case GATT_BATTERY_NOW_REQ:
                    strcpy(message, "GATT_BATTERY_NOW_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_BATTERY_NOW_RES;
                    break;

                    //********************************************************************************
                case GATT_BATTERY_NOW_RES:

                    buf[2] = 0x06; //length
                    buf[3] = 0x00;
                    buf[4] = 0x02;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x64;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_BATTERY_DESC_REQ;
                    break;

                case GATT_BATTERY_DESC_REQ:
                    strcpy(message, "GATT_BATTERY_DESC_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_BATTERY_DESC_RES;
                    break;

                    //********************************************************************************
                case GATT_BATTERY_DESC_RES:

                    buf[2] = 0x0a; //length
                    buf[3] = 0x00;
                    buf[4] = 0x06;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x05;
                    buf[9] = 0x01;
                    buf[10] = 0x19;
                    buf[11] = 0x00;
                    buf[12] = 0x02;
                    buf[13] = 0x29;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_BATTERY_CCC_REQ;
                    break;

                case GATT_BATTERY_CCC_REQ:
                    strcpy(message, "GATT_BATTERY_CCC_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_BATTERY_CCC_RES;
                    break;

                    //********************************************************************************
                case GATT_BATTERY_CCC_RES:
                    buf[2] = 0x07; //length
                    buf[3] = 0x00;
                    buf[4] = 0x03;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x01;
                    buf[10] = 0x00;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_INDUCED_SERVICE;
                    break;

                case GATT_HID_INDUCED_SERVICE:
                    strcpy(message, "GATT_HID_INDUCED_SERVICE:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_INDUCED_SERVICE_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_INDUCED_SERVICE_RES:

                    buf[2] = 0x09;
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x01;
                    buf[9] = 0x08;
                    buf[10] = 0x06;
                    buf[11] = 0x00;
                    buf[12] = 0x0a;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_CHAR1_REQ;
                    break;

                case GATT_HID_CHAR1_REQ:
                    strcpy(message, "GATT_HID_CHAR1_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_CHAR1_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_CHAR1_RES:

                    buf[2] = 0x1b; //length
                    buf[3] = 0x00;
                    buf[4] = 0x17;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x09;
                    buf[9] = 0x07;
                    for (step = 0; step < 7; step++) {
                        buf[10 + step] = HID_CHAR[0][step];
                        buf[17 + step] = HID_CHAR[1][step];
                        buf[24 + step] = HID_CHAR[2][step];
                    }

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_CHAR2_REQ;
                    break;

                case GATT_HID_CHAR2_REQ:
                    strcpy(message, "GATT_HID_CHAR2_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_CHAR2_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_CHAR2_RES:

                    buf[2] = 0x1b; //length
                    buf[3] = 0x00;
                    buf[4] = 0x17;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x09;
                    buf[9] = 0x07;
                    for (step = 0; step < 7; step++) {
                        buf[10 + step] = HID_CHAR[3][step];
                        buf[17 + step] = HID_CHAR[4][step];
                        buf[24 + step] = HID_CHAR[5][step];
                    }

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_CHAR3_REQ;
                    break;

                case GATT_HID_CHAR3_REQ:
                    strcpy(message, "GATT_HID_CHAR3_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_CHAR4_RES;
                    break;

                    //********************************************************************************
                    /*
                           case  GATT_HID_CHAR3_RES:

                               buf[2]=0x0d;//length
                               buf[3]=0x00;
                               buf[4]=0x09;
                               buf[5]=0x00;
                               buf[6]=0x04;
                               buf[7]=0x00;
                               buf[8]=0x09;
                               buf[9]=0x07;
                               for(step=0;step<7;step++) {
                                   buf[10+step]=HID_CHAR[6][step];
                               }

                               DemoState = BT_STATE_WRITE_ACL;
                               HciState = GATT_HID_CHAR4_REQ;
                               break;

                           case GATT_HID_CHAR4_REQ:
                               strcpy(message,"GATT_HID_CHAR4_REQ:\r\n");
                               DemoState = BT_STATE_READ_ACL_HCI;
                               HciState = GATT_HID_CHAR4_RES;
                               break;
                     */
                    //********************************************************************************
                case GATT_HID_CHAR4_RES:

                    buf[2] = 0x09; //length
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x01;
                    for (step = 0; step < 4; step++) buf[9 + step] = HID_CHAR[6][step];

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_INFO_REQ;
                    break;

                case GATT_HID_INFO_REQ:
                    strcpy(message, "GATT_HID_INFO_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_INFO_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_INFO_RES:
                    buf[2] = 0x09; //length
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x02;
                    buf[10] = 0x00;
                    buf[11] = 0x00;
                    buf[12] = 0x10;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_BOOT_IN_REQ;
                    break;

                    //		case GATT_HID_BOOT_OUT_REQ:
                    //			strcpy(message,"GATT_HID_BOOT_OUT_REQ:\r\n");
                    //			DemoState = BT_STATE_READ_ACL_HCI;
                    //			HciState = GATT_HID_BOOT_OUT_RES;
                    //			break;

                    //********************************************************************************
                    /*
                                case GATT_HID_BOOT_OUT_RES:
                                buf[2]=0x06;//length
                                buf[3]=0x00;
                                buf[4]=0x02;
                                buf[5]=0x00;
                                buf[6]=0x04;
                                buf[7]=0x00;
                                buf[8]=0x0b;
                                buf[9]=0x00;//???????????????????????

                                DemoState = BT_STATE_WRITE_ACL;
                                HciState = GATT_HID_BOOT_IN_REQ;
                                break;
                     */
                case GATT_HID_BOOT_IN_REQ:
                    strcpy(message, "GATT_HID_BOOT_IN_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_BOOT_IN_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_BOOT_IN_RES:
                    buf[2] = 0x06; //length
                    buf[3] = 0x00;
                    buf[4] = 0x02;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x00; //???????????????????????

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_DESC1_REQ;
                    break;

                case GATT_HID_DESC1_REQ:
                    strcpy(message, "GATT_HID_DESC1_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_DESC1_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_DESC1_RES:

                    buf[2] = 0x0a; //length
                    buf[3] = 0x00;
                    buf[4] = 0x06;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x05;
                    buf[9] = 0x01;
                    buf[10] = 0x11;
                    buf[11] = 0x00;
                    buf[12] = 0x02;
                    buf[13] = 0x29;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_CCC1_REQ;
                    break;

                case GATT_HID_CCC1_REQ:
                    strcpy(message, "GATT_HID_CCC1_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_CCC1_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_CCC1_RES:

                    buf[2] = 0x07; //length
                    buf[3] = 0x00;
                    buf[4] = 0x03;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x01;
                    buf[10] = 0x00;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_REPORT1_REQ;
                    break;

                case GATT_HID_REPORT1_REQ:
                    strcpy(message, "GATT_HID_REPORT1_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_REPORT1_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_REPORT1_RES:

                    buf[2] = 0x1b; //length
                    buf[3] = 0x00;
                    buf[4] = 0x17;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    for (step = 0; step < 22; step++) buf[9 + step] = HID_REPORT[0][step];

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_REPORT2_REQ;
                    break;

                case GATT_HID_REPORT2_REQ:
                    strcpy(message, "GATT_HID_REPORT2_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_REPORT2_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_REPORT2_RES:

                    buf[2] = 0x1b; //length
                    buf[3] = 0x00;
                    buf[4] = 0x17;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0d;
                    for (step = 0; step < 22; step++) buf[9 + step] = HID_REPORT[1][step];

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_REPORT_REQ3;
                    break;

                case GATT_HID_REPORT_REQ3:
                    strcpy(message, "GATT_HID_REPORT_REQ3:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_REPORT_RES3;
                    break;

                    //********************************************************************************
                case GATT_HID_REPORT_RES3:

                    buf[2] = 0x0b; //length
                    buf[3] = 0x00;
                    buf[4] = 0x07;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0d;
                    for (step = 0; step < 6; step++) buf[9 + step] = HID_REPORT[2][step];

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_DATA_REQ;
                    break;

                case GATT_HID_DATA_REQ:
                    strcpy(message, "GATT_HID_DATA_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_DATA_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_DATA_RES:

                    buf[2] = 0x06; //length
                    buf[3] = 0x00;
                    buf[4] = 0x02;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x00; //???????????????????????

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_DESC2_REQ;
                    break;

                case GATT_HID_DESC2_REQ:
                    strcpy(message, "GATT_HID_DESC2_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_DESC2_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_DESC2_RES:

                    buf[2] = 0x0e; //length
                    buf[3] = 0x00;
                    buf[4] = 0x0a;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x05;
                    buf[9] = 0x01;
                    buf[10] = 0x0b;
                    buf[11] = 0x00;
                    buf[12] = 0x08;
                    buf[13] = 0x29;
                    buf[14] = 0x0c;
                    buf[15] = 0x00;
                    buf[16] = 0x02;
                    buf[17] = 0x29;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_HID_CCC2_REQ;
                    break;

                case GATT_HID_CCC2_REQ:
                    strcpy(message, "GATT_HID_CCC2_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_HID_CCC2_RES;
                    break;

                    //********************************************************************************
                case GATT_HID_CCC2_RES:

                    buf[2] = 0x07; //length
                    buf[3] = 0x00;
                    buf[4] = 0x03;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x01;
                    buf[10] = 0x00;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_REPORT_ID_IN_REQ;
                    break;

                case GATT_REPORT_ID_IN_REQ:
                    strcpy(message, "GATT_REPORT_ID_IN_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_REPORT_ID_IN_RES;
                    break;

                    //********************************************************************************
                case GATT_REPORT_ID_IN_RES:

                    buf[2] = 0x07; //length
                    buf[3] = 0x00;
                    buf[4] = 0x03;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x00;
                    buf[10] = 0x01;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_PROT_MODE_REQ;
                    break;

                case GATT_PROT_MODE_REQ:
                    strcpy(message, "GATT_PROT_MODE_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_PROT_MODE_RES;
                    break;

                    //********************************************************************************
                case GATT_PROT_MODE_RES:
                    buf[2] = 0x06; //length
                    buf[3] = 0x00;
                    buf[4] = 0x02;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x01;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_ACCESS_INDUCED_SERVICE;
                    break;

                case GATT_ACCESS_INDUCED_SERVICE:
                    strcpy(message, "GATT_ACCESS_INDUCED_SERVICE:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_ACCESS_INDUCED_SERVICE_RES;
                    break;

                    //********************************************************************************
                case GATT_ACCESS_INDUCED_SERVICE_RES:

                    buf[2] = 0x09; //length
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x01;
                    buf[9] = 0x08;
                    buf[10] = 0x01;
                    buf[11] = 0x00;
                    buf[12] = 0x0a;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_ACCESS_CHAR1_REQ;
                    break;

                case GATT_ACCESS_CHAR1_REQ:
                    strcpy(message, "GATT_ACCESS_CHAR1_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_ACCESS_CHAR1_RES;
                    break;

                    //********************************************************************************
                case GATT_ACCESS_CHAR1_RES:

                    buf[2] = 0x14; //length
                    buf[3] = 0x00;
                    buf[4] = 0x10;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x09;
                    buf[9] = 0x07;
                    for (step = 0; step < 7; step++) {
                        buf[10 + step] = ACCESS_SERVICE[0][step];
                        buf[17 + step] = ACCESS_SERVICE[1][step];
                    }

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_ACCESS_CHAR2_REQ;
                    break;

                case GATT_ACCESS_CHAR2_REQ:
                    strcpy(message, "GATT_ACCESS_CHAR2_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_ACCESS_CHAR2_RES;
                    break;

                    //********************************************************************************
                case GATT_ACCESS_CHAR2_RES:

                    buf[2] = 0x09; //length
                    buf[3] = 0x00;
                    buf[4] = 0x05;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x01;
                    for (step = 0; step < 4; step++) buf[9 + step] = ACCESS_SERVICE[2][step];

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_ACCESS_ICON_REQ;
                    break;

                case GATT_ACCESS_ICON_REQ:
                    strcpy(message, "GATT_ACCESS_ICON_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_ACCESS_ICON_RES;
                    break;

                    //********************************************************************************
                case GATT_ACCESS_ICON_RES:

                    buf[2] = 0x07; //length
                    buf[3] = 0x00;
                    buf[4] = 0x03;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0xc2;
                    buf[10] = 0x03;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_ACCESS_NAME_REQ;
                    break;

                case GATT_ACCESS_NAME_REQ:
                    strcpy(message, "GATT_ACCESS_NAME_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_ACCESS_NAME_RES;
                    break;

                    //********************************************************************************
                case GATT_ACCESS_NAME_RES:

                    buf[2] = 0x0b; //length
                    buf[3] = 0x00;
                    buf[4] = 0x07;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 'Y';
                    buf[10] = 'T';
                    buf[11] = 'M';
                    buf[12] = '_';
                    buf[13] = 'L';
                    buf[14] = 'E';

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = GATT_BATTERY_NOW2_REQ;
                    break;

                case GATT_BATTERY_NOW2_REQ:
                    strcpy(message, "GATT_BATTERY_NOW2_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = GATT_BATTERY_NOW2_RES;
                    break;

                    //********************************************************************************
                case GATT_BATTERY_NOW2_RES:

                    buf[2] = 0x06; //length
                    buf[3] = 0x00;
                    buf[4] = 0x02;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x0b;
                    buf[9] = 0x64;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = HID_WRITE_DATA;
                    break;

                    //********************************************************************************
                    //********************************************************************************
                    // SMP LAYER
                    //********************************************************************************
                    //********************************************************************************
                case SMP_PAIR_REQ:
                    strcpy(message, "SMP_PAIR_REQ:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = SMP_PAIR_RES;
                    break;

                case SMP_PAIR_RES:

                    for (step = 0; step < 7; step++) PAIR_REQ[step] = buf[8 + step];

#ifdef DEBUG_MODE
                    for (data_num = 0; data_num < 7; data_num++) {
                        UART2PutHex(PAIR_REQ[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");
#endif

                    buf[2] = 0x0b;
                    buf[3] = 0x00;
                    buf[4] = 0x07;
                    buf[5] = 0x00;
                    buf[6] = 0x06;
                    buf[7] = 0x00;
                    for (step = 0; step < 7; step++) buf[8 + step] = PAIR_RES[step]; //Pairing Response

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = SMP_PAIR_CONF_IN;
                    break;

                    //********************************************************************************
                case SMP_PAIR_CONF_IN:
                    strcpy(message, "SMP_PAIR_CONF_IN:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = SMP_FUNC_E;
                    break;


                    //********************************************************************************
                    //Pair config calculation
                case SMP_FUNC_E:

                    P1[0] = INI_ADDRESS_TYPE;
                    P1[1] = RES_ADDRESS_TYPE;
                    for (step = 0; step < 7; step++) {
                        P1[2 + step] = PAIR_REQ[step];
                        P1[9 + step] = PAIR_RES[step];
                    }

                    buf1[0] = 0x17;
                    buf1[1] = 0x20;
                    buf1[2] = 0x20;
                    for (step = 0; step < 16; step++) buf1[3 + step] = 0x00;
                    for (step = 0; step < 16; step++) buf1[19 + step] = PAIR_RANDOM [step] ^ P1[step];


#ifdef DEBUG_MODE
                    for (data_num = 0; data_num < 35; data_num++) {
                        UART2PutHex(buf1[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");
#endif

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = SMP_FUNC_E_END;
                    break;

                case SMP_FUNC_E_END:
                    end_num = BL_EVENT;
                    strcpy(message, "SMP_FUNC_E_END:\r\n");
                    DemoState = BT_STATE_READ_EP1;
                    HciState = SMP_FUNC_C1;
                    break;

                case SMP_FUNC_C1:

                    for (step = 0; step < 6; step++) {
                        P2[step] = RES_BT_ADDRESS[step];
                        P2[6 + step] = INI_BT_ADDRESS[step];
                    }
                    for (step = 0; step < 4; step++) P2[12 + step] = 0x00;

                    buf1[0] = 0x17;
                    buf1[1] = 0x20;
                    buf1[2] = 0x20;
                    for (step = 0; step < 16; step++) buf1[34 - step] = buf1[21 - step] ^ P2[15 - step];
                    for (step = 0; step < 16; step++) buf1[3 + step] = 0x00;

#ifdef DEBUG_MODE
                    for (data_num = 0; data_num < 35; data_num++) {
                        UART2PutHex(buf1[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");
#endif

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = SMP_FUNC_C1_END;
                    break;

                case SMP_FUNC_C1_END:
                    end_num = BL_EVENT;
                    strcpy(message, "SMP_FUNC_C1_END:\r\n");
                    DemoState = BT_STATE_READ_EP1;
                    HciState = SMP_PAIR_CONF_OUT;
                    break;

                case SMP_PAIR_CONF_OUT:

                    for (step = 0; step < 16; step++) PAIR_CONF[step] = buf1[6 + step];

#ifdef DEBUG_MODE
                    for (data_num = 0; data_num < 16; data_num++) {
                        UART2PutHex(PAIR_CONF[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");
#endif

                    buf[2] = 0x15;
                    buf[3] = 0x00;
                    buf[4] = 0x11;
                    buf[5] = 0x00;
                    buf[6] = 0x06;
                    buf[7] = 0x00;
                    buf[8] = 0x03; //Pairing Confirm
                    for (step = 0; step < 16; step++) buf[9 + step] = PAIR_CONF[step];

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = SMP_RANDOM_IN;
                    break;

                    //********************************************************************************
                case SMP_RANDOM_IN:
                    strcpy(message, "SMP_RANDOM_IN:\r\n");
                    DemoState = BT_STATE_READ_ACL_HCI;
                    HciState = SMP_RANDOM_OUT;
                    break;

                case SMP_RANDOM_OUT:

                    for (step = 0; step < 16; step++) INIT_RANDOM[step] = buf[9 + step];

#ifdef DEBUG_MODE
                    for (data_num = 0; data_num < 16; data_num++) {
                        UART2PutHex(INIT_RANDOM[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");
#endif

                    buf[2] = 0x15;
                    buf[3] = 0x00;
                    buf[4] = 0x11;
                    buf[5] = 0x00;
                    buf[6] = 0x06;
                    buf[7] = 0x00;
                    buf[8] = 0x04; //
                    for (step = 0; step < 16; step++) buf[9 + step] = PAIR_RANDOM[step];

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = SMP_FUNC_S1;
                    break;

                    //********************************************************************************
                    //STK = s1(TK, Srand, Mrand)
                case SMP_FUNC_S1:

                    buf1[0] = 0x17;
                    buf1[1] = 0x20;
                    buf1[2] = 0x20;
                    for (step = 0; step < 16; step++) buf1[3 + step] = 0x00;
                    for (step = 0; step < 8; step++) {
                        buf1[19 + step] = INIT_RANDOM[step];
                        buf1[27 + step] = PAIR_RANDOM[step];
                    }

#ifdef DEBUG_MODE
                    for (data_num = 0; data_num < 35; data_num++) {
                        UART2PutHex(buf1[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");
#endif

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = SMP_FUNC_S1_END;
                    break;

                case SMP_FUNC_S1_END:
                    end_num = BL_EVENT;
                    strcpy(message, "SMP_FUNC_S1_END:\r\n");
                    DemoState = BT_STATE_READ_EP1;
                    HciState = SMP_LTK_REQ;
                    break;

                    //********************************************************************************
                case SMP_LTK_REQ:

                    for (step = 0; step < 16; step++) STK[step] = buf1[6 + step];

                    end_num = BLE_EVENT;
                    strcpy(message, "SMP_LTK_REQ:\r\n");
                    DemoState = BT_STATE_READ_EP1;
                    HciState = SMP_LTK_RES;
                    break;

                    //********************************************************************************
                case SMP_LTK_RES:

                    buf1[0] = 0x1a;
                    buf1[1] = 0x20;
                    buf1[2] = 0x12;
                    //			buf1[3]=0x00;
                    //			buf1[4]=0x00;
                    for (step = 0; step < 16; step++) buf1[5 + step] = STK[step];

#ifdef DEBUG_MODE
                    for (data_num = 0; data_num < 21; data_num++) {
                        UART2PutHex(buf1[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");
#endif

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = SMP_LTK_RES_END;
                    break;


                case SMP_LTK_RES_END:

                    end_num = BL_EVENT;//ENCRYPT_EVENT;
                    strcpy(message, "SMP_LTK_RES_END:\r\n");
                    DemoState = BT_STATE_READ_EP1;
                    HciState = SMP_LTK_OUT;
                    break;

                    //********************************************************************************
                    //********************************************************************************
                    //********************************************************************************
                    //********************************************************************************
                case SMP_LTK_OUT:

                    buf[2] = 0x15;
                    buf[3] = 0x00;
                    buf[4] = 0x11;
                    buf[5] = 0x00;
                    buf[6] = 0x06;
                    buf[7] = 0x00;
                    buf[8] = 0x06; //
                    for (step = 0; step < 16; step++) buf[9 + step] = LTK[step];

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = SMP_RANDOM1_OUT;
                    break;

                case SMP_RANDOM1_OUT:

                    buf[2] = 0x0f;
                    buf[3] = 0x00;
                    buf[4] = 0x0b;
                    buf[5] = 0x00;
                    buf[6] = 0x06;
                    buf[7] = 0x00;
                    buf[8] = 0x07; //
                    for (step = 0; step < 10; step++) buf[9 + step] = EDIV_RANDOM[step];

                    DemoState = BT_STATE_WRITE_ACL;
                    //YTS			HciState = SMP_LTK_IN;
                    HciState = GATT_MTU_RES;
                    break;
                    //return to GATT

                    /*
                    case  SMP_LTK_IN:

                        strcpy(message,"SMP_LTK_IN:\r\n");
                        DemoState = BT_STATE_READ_ACL_HCI;
                        HciState = SMP_RANDOM1_IN;
                        break;

                    case  SMP_RANDOM1_IN:
                        for(step=0;step<10;step++) M_EDIV_RANDOM[step]=buf[9+step];
                        strcpy(message,"SMP_RANDOM1_IN:\r\n");
                        DemoState = BT_STATE_READ_ACL_HCI;
                        HciState = GATT_HID_CHAR1_REQ;
                        break;
                     */

                    //********************************************************************************
                case L2CAP_DISCONNECT_REQ:
                    buf1[0] = 0x06;
                    buf1[1] = 0x04;
                    buf1[2] = 0x03;
                    buf1[3] = handle[0];
                    buf1[4] = handle[1];
                    buf1[5] = 0x13;

#ifdef DEBUG_MODE
                    UART2PrintString("DISCONNECT_REQUEST: ");
                    for (data_num = 0; data_num < buf1[1] + 2; data_num++) {
                        UART2PutHex(buf1[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString("\r\n");
#endif

                    DemoState = BT_STATE_WRITE_CLASS;
                    HciState = L2CAP_DISCONNECT_REQ_END;
                    break;


                case L2CAP_DISCONNECT_REQ_END:

                    strcpy(message, "");
                    LED_Off();
                    end_num = 0x0f; //DISCONNECT;
                    DemoState = BT_STATE_READ_EP1;
                    HciState = PROG_END;
                    break;


                    //********************************************************************************
                    //********************************************************************************
                    // HOGP (HID OVER GATT PROFILE)
                    //********************************************************************************
                    //********************************************************************************

                case HID_WRITE_DATA:
                    LED_On();

                    if (movement_length > 14) {
                        mouse[1] = dir_table[vector & 0x07]; // X-Vector
                        mouse[2] = dir_table[(vector + 2) & 0x07]; // Y-Vector
                        //go to the next direction in the table
                        vector++;
                        //reset the counter for when to change again
                        movement_length = 0;
                    }

                    buf[2] = 0x0a;
                    buf[3] = 0x00;
                    buf[4] = 0x06;
                    buf[5] = 0x00;
                    buf[6] = 0x04;
                    buf[7] = 0x00;
                    buf[8] = 0x1b;
                    buf[9] = 0x0a;
                    buf[10] = 0x00;
                    buf[11] = 0; //Button and Padding
                    buf[12] = mouse[1]; // X-Vector
                    buf[13] = mouse[2]; // Y-Vector

                    movement_length++;

                    DemoState = BT_STATE_WRITE_ACL;
                    HciState = HID_READ_HCI;

                    break;

                case HID_READ_HCI:
                    DemoState = BT_STATE_READ_HCI;
                    HciState = HID_WRITE_DATA;
                    break;

                case PROG_END:
                    break;

            }
            break;

            //********************************************************************************
            //?READ-WRITE FUNCTIONS
            //********************************************************************************
            //WRITE ENDPOINT 0
        case BT_STATE_WRITE_CLASS:
            if (!USBHostGenericTxIsBusy(deviceAddress)) {
                if ((RetVal = USBHostGenericClassRequest(deviceAddress, buf1, buf1[2] + 3)) == USB_SUCCESS) {
                    //UART2PrintString( "HCI COMMAND SENT\r\n" );
                    DemoState = BT_STATE_PROCESS;
                } else {
                    UART2PrintString("Write Class Error !\r\n");
                }
            }
            break;


            //READ ENDPOINT 1
        case BT_STATE_READ_EP1:
            if (!USBHostGenericRx1IsBusy(deviceAddress)) {
                if ((RetVal = USBHostGenericRead(deviceAddress, buf1, MAX_NO_OF_BYTES)) == USB_SUCCESS) {
                    DemoState = BT_STATE_READ_CLASS_WAITING;
                    //UART2PrintString( "READ EP1\r\n" );
                } else {
                    UART2PrintString("Device Read Error !\r\n");
                }
            }
            break;

        case BT_STATE_READ_CLASS_WAITING:
            if (!USBHostGenericRx1IsBusy(deviceAddress)) {

#ifdef DEBUG_MODE
                UART2PrintString(message);
                for (data_num = 0; data_num < buf1[1] + 2; data_num++) {
                    UART2PutHex(buf1[data_num]);
                    UART2PutChar(' ');
                }
                UART2PrintString("\r\n");
#endif

                if (buf1[0] != end_num) {
                    DemoState = BT_STATE_READ_EP1;
                    break;
                }

                DemoState = BT_STATE_PROCESS;
            }
            break;


            //WRITE ENDPOINT 2
        case BT_STATE_WRITE_ACL:

#ifdef DEBUG_MODE
            for (data_num = 0; data_num < buf[2] + 4; data_num++) {
                UART2PutHex(buf[data_num]);
                UART2PutChar(' ');
            }
            UART2PrintString("\r\n");
#endif

            if (!USBHostGenericTxIsBusy(deviceAddress)) {
                if ((RetVal = USBHostGenericAclWrite(deviceAddress, buf, buf[2] + 4)) == USB_SUCCESS) {
                    //UART2PrintString( "HCI COMMAND SENT\r\n" );
                    DemoState = BT_STATE_PROCESS;
                } else {
                    UART2PrintString("Write Acl Error !\r\n");
                }
            }
            break;

            //READ ENDPOINT 2 & 1
        case BT_STATE_READ_ACL_HCI:
            if (EP2busy_flag == 1) {
                DemoState = BT_STATE_READ_ACL_WAITING;
                break;
            }
            if (!USBHostGenericRx2IsBusy(deviceAddress)) {
                if ((RetVal = USBHostGenericAclRead(deviceAddress, buf, MAX_NO_OF_BYTES)) == USB_SUCCESS) {
                    DemoState = BT_STATE_READ_ACL_WAITING;
                } else {
                    UART2PrintString("Read Acl Error !\r\n");
                }
            }
            break;

        case BT_STATE_READ_ACL_WAITING:
            if (!USBHostGenericRx2IsBusy(deviceAddress)) {

#ifdef DEBUG_MODE
                UART2PrintString(message);
                for (data_num = 0; data_num < buf[2] + 4; data_num++) {
                    UART2PutHex(buf[data_num]);
                    UART2PutChar(' ');
                }
                UART2PrintString("\r\n");
#endif
                EP2busy_flag = 0;
                DemoState = BT_STATE_READ_HCI;
            } else {
                EP2busy_flag = 1;
                DemoState = BT_STATE_READ_HCI;
            }
            break;

        case BT_STATE_READ_HCI:
            if (EP2busy_flag == 0) buf1[0] = 0xff;
            if (!USBHostGenericRx1IsBusy(deviceAddress)) {
                if ((RetVal = USBHostGenericRead(deviceAddress, buf1, MAX_NO_OF_BYTES)) == USB_SUCCESS) {
                    DemoState = BT_STATE_READ_HCI_WAITING;
                    //UART2PrintString( "READ EP1\r\n" );
                } else {
                    UART2PrintString("Device Read Error !\r\n");
                }
            }
            break;

        case BT_STATE_READ_HCI_WAITING:
            if (!USBHostGenericRx1IsBusy(deviceAddress)) {

#ifdef DEBUG_MODE
                UART2PrintString("HCI: ");
                for (data_num = 0; data_num < buf1[1] + 2; data_num++) {
                    UART2PutHex(buf1[data_num]);
                    UART2PutChar(' ');
                }
                UART2PrintString("\r\n");
#endif

                if (buf1[0] == DISCONNECT) {
                    HciState = L2CAP_DISCONNECT_REQ;
                    DemoState = BT_STATE_PROCESS;
                    break;
                }// following code is not needed in principle, but it is needed in practice in debug mode
                else if (buf1[0] == BLE_EVENT && buf1[1] == 0x0d) {
                    HciState = HCI_CMD_RESET;
                    DemoState = BT_STATE_PROCESS;
                    break;
                }

                if (EP2busy_flag == 0) {
                    if (buf1[0] == 0xff) {
                        DemoState = BT_STATE_PROCESS;
                    } else {
                        DemoState = BT_STATE_READ_HCI;
                    }
                } else {
                    //				if(buf1[0]==DISCONNECT){HciState = L2CAP_DISCONNECT_REQ;DemoState = BT_STATE_PROCESS;}
                    //              else if(buf1[0]==BLE_EVENT && buf1[1]==0x0d){HciState = LTK_REQ_REPLY;DemoState = BT_STATE_PROCESS;}
                    //				else {DemoState = BT_STATE_READ_ACL_WAITING;}
                    DemoState = BT_STATE_READ_ACL_WAITING;
                }
            }
            break;

            //********************************************************************************

            /** Error state:  Hold here until detached **/
        case BT_STATE_ERROR: // To Do: Flash LEDs
            break;

        default:
            DemoState = BT_INITIALIZE;
            break;
    }

    DelayMs(1); // 1ms delay

} // ManageDemoState

/*
void __attribute__((interrupt, no_auto_psv)) _INT1Interrupt(void) {

    DemoState = BT_STATE_PROCESS;
    HciState = L2CAP_DISCONNECT_REQ;
    strcpy(message, "DISCONNECT:\r\n");
    IFS1bits.INT1IF = 0;
}
*/

//******************************************************************************
//******************************************************************************
// USB Support Functions
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        USB_ApplicationEventHandler
 *
 * Preconditions:   The USB must be initialized.
 *
 * Input:           event       Identifies the bus event that occured
 *
 *                  data        Pointer to event-specific data
 *
 *                  size        Size of the event-specific data
 *
 * Output:          deviceAddress (global)
 *                  Updates device address when an attach or detach occurs.
 *
 *                  DemoState (global)
 *                  Updates the demo state as appropriate when events occur.
 *
 * Returns:         true if the event was handled, false if not
 *
 * Side Effects:    Event-specific actions have been taken.
 *
 * Overview:        This routine is called by the Host layer or client
 *                  driver to notify the application of events that occur.
 *                  If the event is recognized, it is handled and the
 *                  routine returns true.  Otherwise, it is ignored (or
 *                  just "sniffed" and the routine returns false.
 *************************************************************************/

bool USB_ApplicationEventHandler(uint8_t address, USB_EVENT event, void *data, uint32_t size) {
#ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
    uint8_t i;
#endif

    // Handle specific events.
    switch ((int)event) {
        case EVENT_GENERIC_ATTACH:
            if (size == sizeof (GENERIC_DEVICE_ID)) {
                deviceAddress = ((GENERIC_DEVICE_ID *) data)->deviceAddress;
                DemoState = BT_STATE_PROCESS;
                HciState = HCI_CMD_RESET; //YTS !!!!!!!!!!!!!!!!!
                UART2PrintString("Generic demo device attached - event, deviceAddress=");
                UART2PutDec(deviceAddress);
                UART2PrintString("\r\n");
#ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
                for (i = 1; i < ((GENERIC_DEVICE_ID *) data)->serialNumberLength; i++) {
                    UART2PutChar(((GENERIC_DEVICE_ID *) data)->serialNumber[i]);
                }
#endif
                UART2PrintString("\r\n");
                return true;
            }
            break;

        case EVENT_GENERIC_DETACH:
            deviceAddress = 0;
            DemoState = BT_INITIALIZE; //YTS
            UART2PrintString("Generic demo device detached - event\r\n");
            return true;

        case EVENT_GENERIC_TX_DONE: // The main state machine will poll the driver.
        case EVENT_GENERIC_RX1_DONE: //YTS
        case EVENT_GENERIC_RX2_DONE: //YTS
            return true;

        case EVENT_VBUS_REQUEST_POWER:
            // We'll let anything attach.
            return true;

        case EVENT_VBUS_RELEASE_POWER:
            // We aren't keeping track of power.
            return true;

        case EVENT_HUB_ATTACH:
            UART2PrintString("\r\n***** USB Error - hubs are not supported *****\r\n");
            return true;
            break;

        case EVENT_UNSUPPORTED_DEVICE:
            UART2PrintString("\r\n***** USB Error - device is not supported *****\r\n");
            return true;
            break;

        case EVENT_CANNOT_ENUMERATE:
            UART2PrintString("\r\n***** USB Error - cannot enumerate device *****\r\n");
            return true;
            break;

        case EVENT_CLIENT_INIT_ERROR:
            UART2PrintString("\r\n***** USB Error - client driver initialization error *****\r\n");
            return true;
            break;

        case EVENT_OUT_OF_MEMORY:
            UART2PrintString("\r\n***** USB Error - out of heap memory *****\r\n");
            return true;
            break;

        case EVENT_UNSPECIFIED_ERROR: // This should never be generated.
            UART2PrintString("\r\n***** USB Error - unspecified *****\r\n");
            return true;
            break;

        case EVENT_SUSPEND:
        case EVENT_DETACH:
        case EVENT_RESUME:
        case EVENT_BUS_ERROR:
            return true;
            break;

        default:
            break;
    }

    return false;

} // USB_ApplicationEventHandler


//******************************************************************************
//******************************************************************************
// Main
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        main
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         Never exits
 *
 * Side Effects:    Runs the application
 *
 * Overview:        This is the USB Custom Demo Application's main entry
 *                  point.
 *************************************************************************/

int main(void) {

    // Initialize the processor and peripherals.
    SYSTEM_Initialize();

    // Set Default demo state
    DemoState = BT_INITIALIZE;
    HciState = HCI_CMD_RESET;

    LED_Off();

    // Main Processing Loop
    while (1) {
        // This demo does not check for overcurrent conditions.  See the
        // USB Host - Data Logger for an example of overcurrent detection
        // with the PIC24F and the USB PICtail Plus.

        // Maintain USB Host State
        USBHostTasks();
        // Maintain Demo Application State
        ManageDemoState();
    }

    return 0;

} // main


/*************************************************************************
 * EOF main.c
 */


